/* main.c - The guts of the Free-DOS Install program. */

/* Copyright (C) 1997, James Hall <jhall1@isd.net> */

/* This is free software.  See the file COPYING for details */


#include <stdio.h>
#include <stdlib.h>				/* for free() */
#include <unistd.h>				/* for getopt() */

#include "cat.h"
#include "dat.h"
#include "getkey.h"
#include "getsmatch.h"
#include "inst.h"
#include "pause.h"
#include "showdesc.h"


/* Symbolic constants */

#define STRLEN 256

#define DAT_FILE  "INSTALL.DAT"			/* disk sets to install */
#define DESC_FILE "INSTALL.TXT"			/* descriptions of disk sets */

#define EXTRACTOR_FILE "EXTRACT.LST"		/* list of file extractors */
#define OEM_FILE  "OEM.TXT"			/* the vendor file */


/* Globals */

extern unsigned g_errs;				/* inst.c */
extern unsigned g_warns;			/* inst.c */

extern char *g_macro;				/* inst.c */


int
main(int argc, char *argv[])
{
  char from[STRLEN];				/* where to install from */
  char exdir[STRLEN];				/* where to install to */
  int i;

  int dsk_ser_n;				/* Size of disk series ary */
  dat_t *dsk_ser;				/* The disk series array */


  /* Evaluate the command line. */

  while ((i = getopt (argc, argv, "x:")) != EOF) {
    switch (i) {
    case 'x':					/* extractor */
#if !defined(unix)
    case 'X':					/* act case insensitively */
#endif

      /* Now see if the extractor program given on the command line
         appears in our extractors list. */

      g_macro = getsmatch (EXTRACTOR_FILE, optarg);
      if (g_macro == NULL) {
	/* No valid extractor found! */

	fprintf (stderr, "install: garbled or unknown extractor program\n");
	exit (2);
      } /* if */
      break;

    default:					/* what? */
#if defined(unix)
      fprintf (stderr, "USAGE: %s [-x extractor_program]\n", argv[0]);
#else
      fprintf (stderr, "USAGE: install [/x extractor_program]\n");
#endif

      fprintf (stderr, "  extractor_program is an external program that exists in your PATH that can\n");
      fprintf (stderr, "  unpack a Package File.  You may NOT specify a path for the program.\n");

      if (optopt != '?') {
	/* "/?" means to display usage for the program. */
	fprintf (stderr, "install: unrecognized option on command line\n");
	exit (1);
      }

      else {
	exit (0);
      }

      break;
    } /* switch */
  } /* while */

  if (optind < argc) {
    /* There are options left, but there shouldn't be! */

    fprintf (stderr, "install: too many options on command line\n");
    exit (1);
  }


  /* Say hello.  OEM_FILE is a convenience for the vendor, so I really
     don't care if 'cat' succeeds or not. */

  printf ("-- Free-DOS Install --\n");
  cat(OEM_FILE);

  printf ("\nCopyright (C) 1997, James Hall <jhall1@isd.net>\n");
  printf ("This is free software.  See the file COPYING for details.\n\n");

  /* Read the package list */

  if (!(dsk_ser = dat_read (DAT_FILE))) {
    /* Failed */

    fprintf (stderr, "install: Can't find %s\n", DAT_FILE);
    exit (1);
  }

  dsk_ser_n = dat_ary_size;

  /* Find out where to install from/to */

  printf ("Install from what path? ");
  if (gets (from) == NULL) {
    strcpy (from, "A:\\");
  }

  printf ("Install files to what path? ");
  if (gets (exdir) == NULL) {
    strcpy (exdir, "C:\\");
  }

  /* Print the package list */

  printf ("\n\n");
  printf ("You are about to install these disk series:\n");

  for (i = 0; i < dsk_ser_n; i++) {
    printf ("%s ", dsk_ser[i].fname);
  }
  printf ("\n\n");

  printf ("The install program will now let you install these disk series.  With\n");
  printf ("each series, you will be able to choose which to install and which to\n");
  printf ("skip.  Each disk series consists of several package files.  You can\n");
  printf ("also select which individual packages to install and which to skip.\n");
  printf ("Note that some disk series and package files may already be selected\n");
  printf ("for you.\n\n");

  /* Allow the user to bail out */

  if (!pause_yn (1)) {
    /* Quit */

    exit (0);
  }

  /* Ask which series to install */

  for (i = 0; i < dsk_ser_n; i++) {
    /* Show the description */

    printf ("\n\n");;
    showdesc (dsk_ser[i].fname, DESC_FILE);

    /* Find out which ones the user wants to install */

    switch (dsk_ser[i].rank) {
    case 'n':
    case 'N':
      /* Do not install */

      printf ("%s [SKIPPED]\n", dsk_ser[i].fname);
      dsk_ser[i].resp = 0;
      break;

    case 'y':
    case 'Y':
      /* Always install */

      printf ("%s [REQUIRED]\n", dsk_ser[i].fname);
      dsk_ser[i].resp = 1;
      break;

    default:
      /* Optional */

      printf ("%s [OPTIONAL]\n", dsk_ser[i].fname);

      /* Ask the user if you want to install it */

      printf ("Install series %s? [yn] ", dsk_ser[i].fname);
      dsk_ser[i].resp = pause_yn (0);
      break;

    } /* switch */
  } /* for */

  /* Run the install */

  printf ("\n\n");
  printf ("Installing the software ...\n");

  for (i = 0; i < dsk_ser_n; i++) {
    if (dsk_ser[i].resp) {
      instser (dsk_ser[i].fname, from, exdir);
    } /* if */
  } /* for */

  /* Done */

  free (dsk_ser);

  /* Print errors and warnings, if any */

  printf ("\n\n");;

  if ((g_errs > 0) || (g_warns > 0)) {
    printf ("There were %u errors and %u warnings.\n", g_errs, g_warns);
  }

  else {
    printf ("The install program completed successfully.\n");
  }

  exit (0);
}
